@@ -10,7 +10,7 @@ from coreir.module import Module, COREModule, COREModule_p, COREModuleDef, COREM
 from coreir.generator import COREGenerator, COREGenerator_p, Generator
 from coreir.namespace import CORENamespace, CORENamespace_p
 from coreir.type import COREType, COREType_p, CoreIRType, Params, Value, Values, COREValue, COREValue_p, Type, NamedType, COREValueType_p, Record
-from coreir.wireable import COREWireable_p, Wireable, inline_instance, connect_const
+from coreir.wireable import COREWireable_p, Wireable, Select, Interface, Instance, inline_instance, cast_to_select, connect_const
 from coreir.type_gen import type_gen, generator_
 from coreir.simulator import SimulatorState, CORESimulatorState_p, CORESimValue_p
 from collections import namedtuple
@@ -60,6 +60,9 @@ libcoreir_c.COREContextRunPasses.restype = ct.c_bool
 
 libcoreir_c.COREPrintErrors.argtypes = [COREContext_p]
 
+libcoreir_c.COREBitInOut.argtypes = [COREContext_p]
+libcoreir_c.COREBitInOut.restype = COREType_p
+
 libcoreir_c.COREBitIn.argtypes = [COREContext_p]
 libcoreir_c.COREBitIn.restype = COREType_p
 
@@ -120,6 +123,19 @@ define_types("COREInlineInstance",[COREWireable_p],ct.c_bool)
 define_types("COREAddPassthrough",[COREWireable_p],COREWireable_p)
 define_types("CORERemoveInstance",[COREWireable_p])
 
+define_types("CORECompileToVerilog",
+             [
+                 COREContext_p,  # context
+                 COREModule_p,  # top
+                 ct.c_char_p,  # filename
+                 ct.c_int,  # num_libs
+                 ct.POINTER(ct.c_char_p),  # libs
+                 ct.c_char_p,  # split
+                 ct.c_char_p,  # product
+                 ct.c_bool,  # inilne
+                 ct.c_bool,  # verilator_debug
+             ], ct.c_bool)
+
 libcoreir_c.COREModuleDefGetInterface.argtypes = [COREModuleDef_p]
 libcoreir_c.COREModuleDefGetInterface.restype = COREWireable_p
 
@@ -141,7 +157,7 @@ libcoreir_c.COREGetModuleRef.restype = COREModule_p
 libcoreir_c.COREGetModArg.argtypes = [COREWireable_p, ct.c_char_p]
 libcoreir_c.COREGetModArg.restype = COREValue_p
 
-libcoreir_c.COREGetModArgs.argtypes = [COREWireable_p, ct.POINTER(ct.POINTER(ct.POINTER(ct.c_char))), ct.POINTER(ct.POINTER(COREValue_p))]
+libcoreir_c.COREGetModArgs.argtypes = [COREWireable_p, ct.POINTER(ct.POINTER(ct.POINTER(ct.c_char))), ct.POINTER(ct.POINTER(COREValue_p)), ct.POINTER(ct.c_int)]
 libcoreir_c.COREGetModArgs.restype = None
 
 libcoreir_c.COREHasModArg.argtypes = [COREWireable_p, ct.c_char_p]
@@ -153,6 +169,9 @@ libcoreir_c.COREGetValueType.restype = ct.c_int
 libcoreir_c.COREValueStringGet.argtypes = [COREValue_p]
 libcoreir_c.COREValueStringGet.restype = ct.c_char_p
 
+libcoreir_c.COREValueJSONGet.argtypes = [COREValue_p]
+libcoreir_c.COREValueJSONGet.restype = ct.c_char_p
+
 libcoreir_c.COREValueIntGet.argtypes = [COREValue_p]
 libcoreir_c.COREValueIntGet.restype = ct.c_int
 
@@ -185,6 +204,9 @@ libcoreir_c.COREValueModule.restype = COREValue_p
 libcoreir_c.COREValueCoreIRType.argtypes = [COREContext_p, COREType_p]
 libcoreir_c.COREValueCoreIRType.restype = COREValue_p
 
+libcoreir_c.COREValueJSON.argtypes = [COREContext_p, ct.c_char_p]
+libcoreir_c.COREValueJSON.restype = COREValue_p
+
 libcoreir_c.COREModuleDefGetConnections.argtypes = [COREModuleDef_p, ct.POINTER(ct.c_int)]
 libcoreir_c.COREModuleDefGetConnections.restype = ct.POINTER(COREConnection_p)
 
@@ -205,6 +227,8 @@ libcoreir_c.COREWireableGetConnectedWireables.restype = ct.POINTER(COREWireable_
 libcoreir_c.COREWireableGetContainer.argtypes = [COREWireable_p]
 libcoreir_c.COREWireableGetContainer.restype = COREModuleDef_p
 
+define_types("COREWireableGetParent",[COREWireable_p], COREWireable_p)
+
 libcoreir_c.COREWireableSelect.argtypes = [COREWireable_p, ct.c_char_p]
 libcoreir_c.COREWireableSelect.restype = COREWireable_p
 
@@ -226,6 +250,7 @@ libcoreir_c.COREModuleDefAddConnectionMetaDataStr.restype = None
 
 libcoreir_c.COREModuleAddMetaDataStr.argtypes = [COREModule_p, ct.c_char_p, ct.c_char_p]
 libcoreir_c.COREModuleAddMetaDataStr.restype = None
+define_types("COREModuleGetMetaData",[COREModule_p],ct.POINTER(ct.c_char))
 
 libcoreir_c.COREModuleDefSelect.argtypes = [COREModuleDef_p, ct.c_char_p]
 libcoreir_c.COREModuleDefSelect.restype = COREWireable_p
