@@ -1,3 +1,4 @@
+import json
 import ctypes as ct
 from coreir.type import COREType_p, Type, Params, COREValue_p, Values, Record
 from coreir.generator import Generator
@@ -61,6 +62,9 @@ class Context:
     def print_errors(self):
         libcoreir_c.COREPrintErrors(self.context)
 
+    def BitInOut(self):
+        return Type(libcoreir_c.COREBitInOut(self.context),self)
+
     def BitIn(self):
         return Type(libcoreir_c.COREBitIn(self.context),self)
 
@@ -121,6 +125,8 @@ class Context:
             elif isinstance(v, coreir.Type):
                 args.append(libcoreir_c.COREValueCoreIRType(self.context,
                     v.ptr))
+            elif isinstance(v, (dict, list)):
+                args.append(libcoreir_c.COREValueJSON(self.context, json.dumps(v).encode()))
             else:
                 raise NotImplementedError(v, type(v))
 
@@ -160,7 +166,7 @@ class Context:
 
     @lru_cache(maxsize=None)
     def get_lib(self, lib):
-        if lib in {"coreir", "mantle", "corebit"}:
+        if lib in {"coreir", "mantle", "corebit", "memory"}:
             return self.get_namespace(lib)
         elif lib == "global":
             return self.global_namespace
@@ -179,6 +185,20 @@ class Context:
                                                 pass_arr, ct.c_int(len(passes)),
                                                 namespaces_arr, ct.c_int(len(namespaces)))
 
+    def compile_to_verilog(self, top, filename, libs=(), split="",
+                           product="", inline=False, verilator_debug=False):
+        top = top.ptr
+        num_libs = ct.c_int(len(libs))
+        libs = (ct.c_char_p * len(libs))(*(lib.encode() for lib in libs))
+        filename = ct.c_char_p(filename.encode())
+        split = ct.c_char_p(split.encode())
+        product = ct.c_char_p(product.encode())
+        inline = ct.c_bool(inline)
+        verilator_debug = ct.c_bool(verilator_debug)
+        return libcoreir_c.CORECompileToVerilog(self.context, top, filename,
+                                                num_libs, libs, split, product,
+                                                inline, verilator_debug)
+
     def __del__(self):
         if not self.external_ptr:
             libcoreir_c.COREDeleteContext(self.context)
